import { visualizationColors } from 'config';
import React, { useRef, useEffect, useState } from 'react';
import Plot from 'react-plotly.js';

export const plotTypeOptions = ['bar', 'scatter', 'line', 'histogram', 'pie'] as const;

export interface CustomChartPlotlyProps {
  data: string[] | number[];
  plotType: (typeof plotTypeOptions)[number];
  title: string;
}

export const CustomChartPlotly: React.FC<CustomChartPlotlyProps> = ({ data, plotType, title }) => {
  const internalRef = useRef<HTMLDivElement>(null);
  const [divSize, setDivSize] = useState({ width: 0, height: 0 });

  useEffect(() => {
    const handleResize = () => {
      if (internalRef.current) {
        const { width, height } = internalRef.current.getBoundingClientRect();
        setDivSize({ width, height });
      }
    };

    handleResize(); // Set initial size
    window.addEventListener('resize', handleResize);
    if (internalRef.current) {
      new ResizeObserver(handleResize).observe(internalRef.current);
    }

    return () => {
      window.removeEventListener('resize', handleResize);
    };
  }, []);

  const getPlotData = (): Partial<Plotly.PlotData>[] => {
    const mainColors = visualizationColors.GPCat.colors[14];
    const xValues = data.map((_, index) => index + 1);

    switch (plotType) {
      case 'bar':
        return [
          {
            type: 'bar',
            x: xValues,
            y: data,
            marker: { color: mainColors },
          },
        ];
      case 'scatter':
        return [
          {
            type: 'scatter',
            x: xValues,
            y: data,
            mode: 'markers',
            marker: { color: mainColors, size: 12 },
          },
        ];
      case 'line':
        return [
          {
            type: 'scatter',
            x: xValues,
            y: data,
            mode: 'lines',
            line: { color: mainColors },
          },
        ];
      case 'histogram':
        return [
          {
            type: 'histogram',
            x: data,
            marker: { color: mainColors },
          },
        ];
      case 'pie':
        return [
          {
            type: 'pie',
            labels: xValues.map(String),
            values: data,
            marker: { colors: mainColors },
          },
        ];
      default:
        return [];
    }
  };

  return (
    <div className="h-full w-full flex items-center justify-center overflow-hidden" ref={internalRef}>
      <Plot
        data={getPlotData()}
        config={{ responsive: true, displayModeBar: false }}
        layout={{
          width: divSize.width,
          height: divSize.height,
          title: title,
          font: {
            family: 'Inter, sans-serif',
            size: 16,
            color: '#374151',
          },
          xaxis: {
            title: 'Category',
          },
          yaxis: {
            title: 'Value',
          },
        }}
      />
    </div>
  );
};
